@@ -122,6 +122,7 @@ route_get(route_t *route, struct route_entry *entry)
 	entry->route_gw.addr_type = ADDR_TYPE_IP;
 	entry->route_gw.addr_bits = IP_ADDR_BITS;
 	entry->route_gw.addr_ip = ipfrow.dwForwardNextHop;
+	entry->metric = ipfrow.dwForwardMetric1;
 
 	entry->intf_name[0] = '\0';
 	intf = intf_open();
@@ -172,6 +173,7 @@ route_loop_getipforwardtable(route_t *r, route_handler callback, void *arg)
 		    &entry.route_dst.addr_bits);
 		entry.route_gw.addr_ip =
 		    r->ipftable->table[i].dwForwardNextHop;
+		entry.metric = r->ipftable->table[i].dwForwardMetric1;
 
 		/* Look up the interface name. */
 		entry.intf_name[0] = '\0';
@@ -209,6 +211,8 @@ route_loop_getipforwardtable2(GETIPFORWARDTABLE2 GetIpForwardTable2,
 	for (i = 0; i < r->ipftable2->NumEntries; i++) {
 		struct intf_entry intf_entry;
 		MIB_IPFORWARD_ROW2 *row;
+		MIB_IPINTERFACE_ROW ifrow;
+		ULONG metric;
 
 		row = &r->ipftable2->Table[i];
 		addr_ston((struct sockaddr *) &row->DestinationPrefix.Prefix, &entry.route_dst);
@@ -223,6 +227,18 @@ route_loop_getipforwardtable2(GETIPFORWARDTABLE2 GetIpForwardTable2,
 		    row->InterfaceIndex) == 0) {
 			strlcpy(entry.intf_name, intf_entry.intf_name, sizeof(entry.intf_name));
 		}
+
+		ifrow.Family = row->DestinationPrefix.Prefix.si_family;
+		ifrow.InterfaceLuid = row->InterfaceLuid;
+		ifrow.InterfaceIndex = row->InterfaceIndex;
+		if (GetIpInterfaceEntry(&ifrow) != NO_ERROR) {
+			return (-1);
+		}
+		metric = ifrow.Metric + row->Metric;
+		if (metric < INT_MAX)
+			entry.metric = metric;
+		else
+			entry.metric = INT_MAX;
 		
 		if ((ret = (*callback)(&entry, arg)) != 0)
 			break;
