@@ -18,6 +18,7 @@
 #include "nse_openssl.h"
 #include "nse_debug.h"
 #include "nse_lpeg.h"
+#include "nse_libssh2.h"
 
 #include <math.h>
 
@@ -545,6 +546,9 @@ static void set_nmap_libraries (lua_State *L)
     {NSE_NMAPLIBNAME, luaopen_nmap},
     {LFSLIBNAME, luaopen_lfs},
     {LPEGLIBNAME, luaopen_lpeg},
+#ifdef HAVE_LIBSSH2
+    {LIBSSH2LIBNAME, luaopen_libssh2},
+#endif
 #ifdef HAVE_OPENSSL
     {OPENSSLLIBNAME, luaopen_openssl},
 #endif
