@@ -12,7 +12,7 @@ _ENV = stdnse.module("ajp", stdnse.seeall)
 -- A basic AJP 1.3 implementation based on documentation available from Apache
 -- mod_proxy_ajp; http://httpd.apache.org/docs/2.2/mod/mod_proxy_ajp.html
 --
--- @author "Patrik Karlsson <patrik@cqure.net>"
+-- @author Patrik Karlsson <patrik@cqure.net>
 --
 
 AJP = {
@@ -531,7 +531,7 @@ Helper = {
 
   -- should only work against 127.0.0.1
   shutdownContainer = function(self)
-    self.comm:send(bin.pack("H", "1234000107"))
+    self.comm:send("\x12\x34\x00\x01\x07")
     self.comm:receive()
   end,
 
