@@ -133,7 +133,7 @@ Options = {
         local parsed_u = url.parse(tostring(u))
 
         if ( o.base_url:getPort() ~= 80 and o.base_url:getPort() ~= 443 ) then
-          if ( tonumber(parsed_u.port) ~= tonumber(o.base_url:getPort()) ) then
+          if ( parsed_u.port ~= tonumber(o.base_url:getPort()) ) then
             return false
           end
         elseif ( parsed_u.scheme ~= o.base_url:getProto() ) then
@@ -149,7 +149,7 @@ Options = {
       o.withindomain = function(u)
         local parsed_u = url.parse(tostring(u))
         if ( o.base_url:getPort() ~= 80 and o.base_url:getPort() ~= 443 ) then
-          if ( tonumber(parsed_u.port) ~= tonumber(o.base_url:getPort()) ) then
+          if ( parsed_u.port ~= tonumber(o.base_url:getPort()) ) then
             return false
           end
         elseif ( parsed_u.scheme ~= o.base_url:getProto() ) then
@@ -233,9 +233,7 @@ LinkExtractor = {
       base_href = base_href .. '/'
     end
 
-    if ( ( base_url:getProto() == 'https' and base_url:getPort() == 443 ) or
-        ( base_url:getProto() == 'http' and base_url:getPort() == 80 ) ) then
-
+    if base_url:getPort() == url.get_default_port(base_url:getProto()) then
       if ( leading_slash ) then
         return ("%s://%s/%s"):format(base_url:getProto(), base_url:getHost(), rel_url)
       else
@@ -427,14 +425,7 @@ URL = {
     self.proto, self.host, self.port, self.file = self.raw:match("^(http[s]?)://([^:/]*)[:]?(%d*)")
     if ( self.proto and self.host ) then
       self.file = self.raw:match("^http[s]?://[^:/]*[:]?%d*(/[^#]*)") or '/'
-      self.port = tonumber(self.port)
-      if ( not(self.port) ) then
-        if ( self.proto:match("https") ) then
-          self.port = 443
-        elseif ( self.proto:match("http")) then
-          self.port = 80
-        end
-      end
+      self.port = tonumber(self.port) or url.get_default_port(self.proto)
 
       self.path  = self.file:match("^([^?]*)[%?]?")
       self.dir   = self.path:match("^(.+%/)") or "/"
@@ -553,7 +544,7 @@ Crawler = {
   iswithinhost = function(self, u)
     local parsed_u = url.parse(tostring(u))
     if ( self.options.base_url:getPort() ~= 80 and self.options.base_url:getPort() ~= 443 ) then
-      if ( tonumber(parsed_u.port) ~= tonumber(self.options.base_url:getPort()) ) then
+      if ( parsed_u.port ~= tonumber(self.options.base_url:getPort()) ) then
         return false
       end
     elseif ( parsed_u.scheme ~= self.options.base_url:getProto() ) then
@@ -570,7 +561,7 @@ Crawler = {
   iswithindomain = function(self, u)
     local parsed_u = url.parse(tostring(u))
     if ( self.options.base_url:getPort() ~= 80 and self.options.base_url:getPort() ~= 443 ) then
-      if ( tonumber(parsed_u.port) ~= tonumber(self.options.base_url:getPort()) ) then
+      if ( parsed_u.port ~= tonumber(self.options.base_url:getPort()) ) then
         return false
       end
     elseif ( parsed_u.scheme ~= self.options.base_url:getProto() ) then
@@ -714,14 +705,36 @@ Crawler = {
   -- Adds a default blacklist blocking binary files such as images,
   -- compressed archives and executable files
   addDefaultBlacklist = function(self)
+    -- References:
+    --[[
+      Image file formats: https://en.wikipedia.org/wiki/Image_file_formats
+      Video file formats: https://en.wikipedia.org/wiki/Video_file_format
+      Audio file formats: https://en.wikipedia.org/wiki/Audio_file_format
+      Doc file extension: https://en.wikipedia.org/wiki/List_of_Microsoft_Office_filename_extensions
+      Archive formats: https://en.wikipedia.org/wiki/List_of_archive_formats ,
+                       https://en.wikipedia.org/wiki/Category:Archive_formats
+    ]]
     local extensions = {
-      image_extensions = {"png","jpg","jpeg","gif","bmp"},
-      video_extensions = {"avi","flv","ogg","mp4","wmv"},
-      audio_extensions = {"aac","m4a","mp3","wav"},
-      doc_extensions = {"pdf", "doc", "docx", "docm", "xls", "xlsx", "xlsm",
-      "ppt", "pptx", "pptm", "odf", "ods", "odp", "ps", "xps"},
+      image_extensions = {"png", "jpg", "jpeg", "gif", "bmp", "jfif", "exif",
+      "tiff", "bmp", "ppm", "pgm", "pbm", "pnm", "webp", "heif", "bpg",
+      "cgm", "svg"},
+      video_extensions = {"avi", "flv", "ogg", "mp4", "m4p", "m4v", "wmv",
+      "vob", "ogv", "mng", "mov", "rmvb", "asf", "nsv", "f4v", "f4p",
+      "amv", "webm", "mkv", "mpg", "mp2", "mpeg", "mpv", "svi", "3gp",
+      "3g2", "mxf", "roq"},
+      audio_extensions = {"aac", "m4a", "mp3", "wav", "aa", "aax", "act", "aiff",
+      "amr", "ape", "au", "awb", "dct", "dss", "dvf", "flac", "gsm", "iklax",
+      "ivs", "m4a", "m4b", "m4p", "mmf", "mpc", "msc", "ogg", "oga", "mogg",
+      "oups", "ra", "raw", "sln", "tta", "vox", "wma", "wv", "webm"},
+      doc_extensions = {"pdf", "doc", "docx", "docm", "xla", "xls", "xlsx",
+      "xlsm", "ppt", "pptx", "pptm", "odf", "ods", "odp", "ps", "xps", "dot",
+      "wbk", "dotx", "dotm", "docb", "xlt", "xlm", "xltx", "xltm", "xlsb",
+      "xlam", "xll", "xlw", "pot", "pps", "potx", "potm", "ppam", "ppsx", "ppsm", "pub"},
       archive_extensions = {"zip", "tar.gz", "gz", "rar", "7z", "sit", "sitx",
-      "tgz", "tar.bz", "tar", "iso"},
+      "tgz", "tar.bz", "tar", "iso", "a", "ar", "cpio", "shar", "lbr", "iso",
+      "mar", "sbx", "bz2", "lz", "lzma", "lzo", "rz", "sz", "s7z", "ace", "afa",
+      "alz", "apk", "tar.bz2", "tar.Z", "tar.lzma", "tlz", "tbz2", "xp3", "zz",
+      "bzip", "lzip", "lzop", "rzip"},
       exe_extensions = {"exe", "com", "msi", "bin","dmg"}
     }
     local blacklist = {}
