@@ -1,16 +1,13 @@
 ---
 -- Utility functions for manipulating and comparing IP addresses.
 --
--- @copyright Same as Nmap--See http://nmap.org/book/man-legal.html
+-- @copyright Same as Nmap--See https://nmap.org/book/man-legal.html
 
-local bin = require "bin"
-local bit = require "bit"
+local math = require "math"
 local stdnse = require "stdnse"
 local string = require "string"
 local table = require "table"
 local type     = type
-local table    = table
-local string   = string
 local ipairs   = ipairs
 local tonumber = tonumber
 local unittest = require "unittest"
@@ -18,6 +15,9 @@ local unittest = require "unittest"
 
 _ENV = stdnse.module("ipOps", stdnse.seeall)
 
+local pack = string.pack
+local unpack = string.unpack
+
 ---
 -- Checks to see if the supplied IP address is part of a non-routable
 -- address space.
@@ -162,16 +162,10 @@ end
 --@return The string representing the address.
 fromdword = function( ip )
   if type( ip ) ~= "number" then
-    stdnse.print_debug(1, "Error in ipOps.todword: Expected IPv4 address.")
+    stdnse.debug1("Error in ipOps.fromdword: Expected 32-bit number.")
     return nil
   end
-
-  local n1 = bit.band(bit.rshift(ip, 0),  0x000000FF)
-  local n2 = bit.band(bit.rshift(ip, 8),  0x000000FF)
-  local n3 = bit.band(bit.rshift(ip, 16), 0x000000FF)
-  local n4 = bit.band(bit.rshift(ip, 24), 0x000000FF)
-
-  return string.format("%d.%d.%d.%d", n1, n2, n3, n4)
+  return string.format("%d.%d.%d.%d", unpack("BBBB", pack(">I4", ip)))
 end
 
 ---
@@ -245,13 +239,9 @@ compare_ip = function( left, op, right )
     return nil, table.concat( err, " " )
   end
 
-  if #left > #right then
-    left = bin.pack( "CA", 0x06, left )
-    right = bin.pack( "CA", 0x04, right )
-  elseif #right > #left then
-    right = bin.pack( "CA", 0x06, right )
-    left = bin.pack( "CA", 0x04, left )
-  end
+  -- by prepending the length, IPv4 (length 4) sorts before IPv6 (length 16)
+  left = pack("s1", left)
+  right = pack("s1", right)
 
   if ( op == "eq" ) then
     return ( left == right )
@@ -509,7 +499,7 @@ get_last_ip = function( ip, prefix )
 end
 
 ---
--- Converts an IP address into an opaque string.
+-- Converts an IP address into an opaque string (big-endian)
 -- @param ip  String representing an IPv4 or IPv6 address.
 -- @param family (optional) Address family to convert to. "ipv6" converts IPv4
 -- addresses to IPv4-mapped IPv6.
@@ -529,12 +519,12 @@ ip_to_str = function( ip, family )
   if not ip:match( ":" ) then
     -- ipv4 string
     for octet in string.gmatch( ip, "%d+" ) do
-      t[#t+1] = bin.pack( ">C", tonumber(octet) )
+      t[#t+1] = pack("B", octet)
     end
   else
     -- ipv6 string
     for hdt in string.gmatch( ip, "%x+" ) do
-      t[#t+1] = bin.pack( ">S", tonumber(hdt, 16) )
+      t[#t+1] = pack( ">I2", tonumber(hdt, 16) )
     end
   end
 
@@ -542,6 +532,26 @@ ip_to_str = function( ip, family )
   return table.concat( t )
 end
 
+---
+-- Converts an opaque string (big-endian) into an IP address
+--
+-- @param ip Opaque string representing an IP address. If length 4, then IPv4
+--           is assumed. If length 16, then IPv6 is assumed.
+-- @return IP address in readable notation (or <code>nil</code> in case of an
+--         error)
+-- @return String error message in case of an error
+str_to_ip = function (ip)
+  if #ip == 4 then
+    return ("%d.%d.%d.%d"):format(unpack("BBBB", ip))
+  elseif #ip == 16 then
+    local full = ("%x:%x:%x:%x:%x:%x:%x:%x"):format(unpack((">I2"):rep(8), ip))
+    full = full:gsub(":[:0]+", "::", 1) -- Collapse the first (should be longest?) series of :0:
+    full = full:gsub("^0::", "::", 1) -- handle special case of ::1
+    return full
+  else
+    return nil, "Invalid length"
+  end
+end
 
 ---
 -- Converts an IP address into a string representing the address as binary
@@ -631,12 +641,15 @@ end
 
 
 
+local bin_lookup = {
+  [0]="0000", "0001", "0010", "0011", "0100", "0101", "0110", "0111",
+      "1000", "1001", "1010", "1011", "1100", "1101", "1110", "1111",
+}
 ---
 -- Converts a string of hexadecimal digits into the corresponding string of
 -- binary digits.
 --
--- Each hex digit results in four bits. This function is really just a wrapper
--- around <code>stdnse.tobinary</code>.
+-- Each hex digit results in four bits.
 -- @param hex  String representing a hexadecimal number.
 -- @usage
 -- bin_string = ipOps.hex_to_bin( "F00D" )
@@ -644,18 +657,48 @@ end
 -- <code>nil</code> in case of an error).
 -- @return     String error message in case of an error.
 hex_to_bin = function( hex )
-
-  if type( hex ) ~= "string" or hex == "" or hex:match( "[^%x]+" ) then
-    return nil, "Error in ipOps.hex_to_bin: Expected string representing a hexadecimal number."
+  if type( hex ) ~= "string" then
+    return nil, "Error in ipOps.hex_to_bin: Expected string"
   end
 
-  local t, mask, binchar = {}, "0000"
-  for hexchar in string.gmatch( hex, "%x" ) do
-      binchar = stdnse.tobinary( tonumber( hexchar, 16 ) )
-      t[#t+1] = mask:sub( 1, # mask  - # binchar  ) .. binchar
+  local status, result = pcall( string.gsub, hex, ".", function(nibble)
+      local n = bin_lookup[tonumber(nibble, 16)]
+      if n then
+        return n
+      else
+        error("Error in ipOps.hex_to_bin: Expected string representing a hexadecimal number.")
+      end
+    end)
+  if status then
+    return result
   end
-  return table.concat( t )
+  return status, result
+end
 
+---
+-- Convert a CIDR subnet mask to dotted decimal notation.
+--
+-- @param subnet CIDR string representing the subnet mask.
+-- @usage
+-- local netmask = ipOps.cidr_to_subnet( "/16" )
+-- @return Dotted decimal representation of the suppliet subnet mask (e.g. "255.255.0.0")
+cidr_to_subnet = function( subnet )
+  local bits = subnet:match("/(%d%d)$")
+  if not bits then return nil end
+  return fromdword((0xFFFFFFFF >> tonumber(bits)) ~ 0xFFFFFFFF)
+end
+
+---
+-- Convert a dotted decimal subnet mask to CIDR notation.
+--
+-- @param subnet Dotted decimal string representing the subnet mask.
+-- @usage
+-- local cidr = ipOps.subnet_to_cidr( "255.255.0.0" )
+-- @return CIDR representation of the supplied subnet mask (e.g. "/16").
+subnet_to_cidr = function( subnet )
+  local dword, err = todword(subnet)
+  if not dword then return nil, err end
+  return "/" .. tostring(32 - (math.tointeger(math.log((dword ~ 0xFFFFFFFF) + 1, 2))))
 end
 
 --Ignore the rest if we are not testing.
@@ -667,7 +710,12 @@ test_suite = unittest.TestSuite:new()
 test_suite:add_test(unittest.is_true(isPrivate("192.168.123.123")), "192.168.123.123 is private")
 test_suite:add_test(unittest.is_false(isPrivate("1.1.1.1")), "1.1.1.1 is not private")
 test_suite:add_test(unittest.equal(todword("65.66.67.68"),0x41424344), "todword")
+test_suite:add_test(unittest.equal(todword("127.0.0.1"),0x7f000001), "todword")
 test_suite:add_test(unittest.equal(fromdword(0xffffffff),"255.255.255.255"), "fromdword")
+test_suite:add_test(unittest.equal(fromdword(0x7f000001),"127.0.0.1"), "fromdword")
+test_suite:add_test(unittest.equal(str_to_ip("\x01\x02\x03\x04"),"1.2.3.4"), "str_to_ip (ipv4)")
+test_suite:add_test(unittest.equal(str_to_ip("\0\x01\xbe\xef\0\0\0\0\0\0\x02\x03\0\0\0\x01"),"1:beef::203:0:1"), "str_to_ip (ipv6)")
+test_suite:add_test(unittest.equal(str_to_ip(("\0"):rep(15) .. "\x01"),"::1"), "str_to_ip (ipv6)")
 test_suite:add_test(function()
   local parts, err = get_parts_as_number("8.255.0.1")
   if parts == nil then return false, err end
@@ -731,6 +779,20 @@ do
   end
 end
 
+do
+  for _, h in ipairs({
+      {"a", "1010"},
+      {"aa", "10101010"},
+      {"12", "00010010"},
+      {"54321", "01010100001100100001"},
+      {"123error", false},
+      {"", ""},
+      {"bad 123", false},
+    }) do
+    test_suite:add_test(unittest.equal(hex_to_bin(h[1]), h[2]))
+  end
+end
+
 do
   for _, op in ipairs({
     {"192.168.13.1", "192/8", unittest.is_true, "IPv4 CIDR"},
@@ -766,5 +828,7 @@ do
   test_suite:add_test(unittest.is_nil(expand_ip("2001:db8::1", "ipv4")),
       "IPv6 to IPv4")
 end
+test_suite:add_test(unittest.equal(cidr_to_subnet("/16"), "255.255.0.0"), "cidr_to_subnet")
+test_suite:add_test(unittest.equal(subnet_to_cidr("255.255.0.0"), "/16"), "subnet_to_cidr")
 
 return _ENV;
