@@ -57,13 +57,6 @@ local function dbg_err(str,...)
 end
 --local err =stdnse.log_error
 
--- Packs data into nullterminated string
---@param input the string to pack
---@return the packed nullterminated string
-local function make_nullterminated_string(input)
-  return bin.pack("z",input)
-end
-
 --Converts an element (key, value) into bson binary data
 --@param key the key name, must *NOT* contain . (period) or start with $
 --@param value, the element value
@@ -138,7 +131,7 @@ function toBson(dict)
   end
   dbg("Packet length is %d",length)
   --Final pack
-  return true, bin.pack("I", length) .. elements .. "\0"
+  return true, bin.pack("<I", length) .. elements .. "\0"
 end
 
 -- Reads a null-terminated string. If length is supplied, it is just cut
@@ -383,7 +376,7 @@ MongoData ={
 --Adds unsigned int32 to the message body
 --@param value the value to add
 function MongoData:addUnsignedInt32(value)
-  self.valueString = self.valueString..bin.pack("I",value)
+  self.valueString = self.valueString..bin.pack("<I",value)
 end
 -- Adds a string to the message body
 --@param value the string to add
@@ -644,42 +637,5 @@ function queryResultToTable( resultTable )
   return result
 
 end
-----------------------------------------------------------------------------------
--- Test-code for debugging purposes below
-----------------------------------------------------------------------------------
-
-
---- Prints data (string) as Hex values, e.g so it more easily can
--- be compared with a packet dump
--- @param strData the data in a string format
-
-local function printBuffer(strData)
-  local out = ''
-  local ch
-  for i  = 1,strData:len() do
-    out = out .." "
-    ch =strData:byte(i)
-    if(ch < 16) then
-      ch = string.format("0%x",ch)
-    else ch = string.format("%x",ch)
-    end
-    --if ch > 64 and ch < 123 then
-    --  out = out .. string.char(ch)
-    --else
-    out = out .. ch
-    --end
-  end
-  print(out)
-end
-
--- function test()
---   local res
---   res = versionQuery()
---   print(type(res),res:len(),res)
---   local  out= bin.unpack('C'..#res,res)
---   printBuffer(res)
--- end
---test()
-
 
 return _ENV;
