@@ -1,11 +1,11 @@
 ---
 -- Simple MySQL Library supporting a very limited subset of operations.
 --
--- http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol
+-- https://dev.mysql.com/doc/internals/en/client-server-protocol.html
 --
--- @copyright Same as Nmap--See http://nmap.org/book/man-legal.html
+-- @copyright Same as Nmap--See https://nmap.org/book/man-legal.html
 --
--- @author "Patrik Karlsson <patrik@cqure.net>"
+-- @author Patrik Karlsson <patrik@cqure.net>
 
 local bin = require "bin"
 local bit = require "bit"
@@ -13,6 +13,7 @@ local nmap = require "nmap"
 local stdnse = require "stdnse"
 local string = require "string"
 local table = require "table"
+local math = require "math"
 _ENV = stdnse.module("mysql", stdnse.seeall)
 
 -- Version 0.3
@@ -50,7 +51,8 @@ Capabilities =
 ExtCapabilities =
 {
   SupportsMultipleStatments = 0x1,
-  SupportsMultipleResults = 0x2
+  SupportsMultipleResults = 0x2,
+  SupportsAuthPlugins = 0x8,
 }
 
 Charset =
@@ -106,7 +108,7 @@ end
 -- <code>status</code> or error message on failure (status == false)
 function receiveGreeting( socket )
 
-  local catch = function() socket:close() stdnse.print_debug("receiveGreeting(): failed") end
+  local catch = function() socket:close() stdnse.debug1("receiveGreeting(): failed") end
   local try = nmap.new_try(catch)
   local data = try( socket:receive_bytes(HEADER_SIZE) )
   local pos, response, tmp, _
@@ -123,23 +125,43 @@ function receiveGreeting( socket )
   pos, is_error = bin.unpack( "C", data, pos )
 
   if ( is_error == 0xff ) then
-    pos, response.errorcode = bin.unpack( "S", data, pos )
+    pos, response.errorcode = bin.unpack( "<S", data, pos )
     pos, response.errormsg = bin.unpack("A" .. (#data - pos + 1), data, pos )
 
     return false, response.errormsg
   end
 
-  pos, response.proto = bin.unpack( "C", data, pos )
+  response.proto = is_error
   pos, response.version = bin.unpack( "z", data, pos )
-  pos, response.threadid = bin.unpack( "I", data, pos )
-  pos, response.salt, _ = bin.unpack( "A8C", data, pos )
-  pos, response.capabilities = bin.unpack( "S", data, pos )
-  pos, response.charset = bin.unpack( "C", data, pos )
-  pos, response.status = bin.unpack( "S", data, pos )
-  pos, _ = bin.unpack( "A13", data, pos )
-  pos, tmp = bin.unpack( "A12", data, pos )
+  pos, response.threadid = bin.unpack( "<I", data, pos )
+
+  if response.proto == 10 then
+    pos, response.salt, _ = bin.unpack( "A8C", data, pos )
+    pos, response.capabilities = bin.unpack( "<S", data, pos )
+    if pos < #data then
+      pos, response.charset = bin.unpack( "C", data, pos )
+      pos, response.status = bin.unpack( "<S", data, pos )
+      pos, response.extcapabilities = bin.unpack( "<S", data, pos) -- capabilities, upper 2 bytes
+      local auth_plugin_len
+      pos, auth_plugin_len = bin.unpack("C", data, pos)
+      pos, tmp = bin.unpack( "A10", data, pos )
+      if tmp ~= "\0\0\0\0\0\0\0\0\0\0" then
+        stdnse.debug2("reserved bytes are not nulls")
+      end
+      if response.capabilities & Capabilities.Support41Auth > 0 then
+        pos, tmp, _ = bin.unpack("A" .. (math.max(13, auth_plugin_len - 8) - 1) .. "x", data, pos)
+        response.salt = response.salt .. tmp
+      end
+      if response.extcapabilities & ExtCapabilities.SupportsAuthPlugins > 0 then
+        response.auth_plugin_name = bin.unpack("z", data, pos)
+      end
+    end
+  elseif response.proto == 9 then
+    pos, response.auth_plugin_data = bin.unpack( "z", data, pos )
+  else
+    stdnse.debug2("Unknown MySQL protocol version: %d", response.proto)
+  end
 
-  response.salt = response.salt .. tmp
   response.errorcode = 0
 
   return true, response
@@ -156,7 +178,7 @@ local function createLoginHash(pass, salt)
   local hash_stage1
   local hash_stage2
   local hash_stage3
-  local reply = ""
+  local reply = {}
   local pos, b1, b2, b3, _ = 1, 0, 0, 0
 
   if ( not(HAVE_SSL) ) then
@@ -171,10 +193,10 @@ local function createLoginHash(pass, salt)
     _, b1 = bin.unpack( "C", hash_stage1, pos )
     _, b2 = bin.unpack( "C", hash_stage3, pos )
 
-    reply = reply .. string.char( bit.bxor( b2, b1 ) )
+    reply[pos] = string.char( bit.bxor( b2, b1 ) )
   end
 
-  return reply
+  return table.concat(reply)
 
 end
 
@@ -193,7 +215,7 @@ end
 -- @return response table or error message on failure
 function loginRequest( socket, params, username, password, salt )
 
-  local catch = function() socket:close() stdnse.print_debug("loginRequest(): failed") end
+  local catch = function() socket:close() stdnse.debug1("loginRequest(): failed") end
   local try = nmap.new_try(catch)
   local packetno = 1
   local authversion = params.authversion or "post41"
@@ -218,20 +240,21 @@ function loginRequest( socket, params, username, password, salt )
   local extcapabilities = ExtCapabilities.SupportsMultipleStatments
   extcapabilities = extcapabilities + ExtCapabilities.SupportsMultipleResults
 
-  local packet = bin.pack( "S", clicap )
-  packet = packet .. bin.pack( "S", extcapabilities )
-  packet = packet .. bin.pack( "I", MAXPACKET )
-  packet = packet .. bin.pack( "C", Charset.latin1_COLLATE_latin1_swedish_ci )
-  packet = packet .. bin.pack( "A", string.char(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0) )
-  packet = packet .. bin.pack( "z", username )
-
+  local hash = ""
   if ( password ~= nil and password:len() > 0 ) then
-    local hash = createLoginHash( password, salt )
-    packet = packet .. bin.pack( "A", string.char( 0x14 ) .. hash )
-  else
-    packet = packet .. bin.pack( "C", 0 )
+    hash = createLoginHash( password, salt )
   end
 
+  local packet = bin.pack( "SSICAzp",
+    clicap,
+    extcapabilities,
+    MAXPACKET,
+    Charset.latin1_COLLATE_latin1_swedish_ci,
+    string.rep("\0", 23),
+    username,
+    hash
+    )
+
   local tmp = packet:len() + bit.lshift( packetno, 24 )
 
   packet = bin.pack( "I", tmp ) .. packet
@@ -322,7 +345,7 @@ function decodeField( data, pos )
 
 end
 
---- Decodes the result set header packet into it's sub components
+--- Decodes the result set header packet into its sub components
 --
 -- ref: http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Result_Set_Header_Packet
 --
@@ -330,7 +353,7 @@ end
 -- @return table containing the following <code>header</code>, <code>fields</code> and <code>data</code>
 function decodeQueryResponse( socket )
 
-  local catch = function() socket:close() stdnse.print_debug("decodeQueryResponse(): failed") end
+  local catch = function() socket:close() stdnse.debug1("decodeQueryResponse(): failed") end
   local try = nmap.new_try(catch)
   local data, header, pos
   local rs, blocks = {}, {}
@@ -350,7 +373,7 @@ function decodeQueryResponse( socket )
   rs.header = data:sub( 1, HEADER_SIZE + header.len )
 
   -- abort on MySQL error
-  if rs.header:sub(HEADER_SIZE + 1, HEADER_SIZE + 1) == string.char(0xFF) then
+  if rs.header:sub(HEADER_SIZE + 1, HEADER_SIZE + 1) == "\xFF" then
     -- is this a 4.0 or 4.1 error message
     if rs.header:find("#") then
       return false, rs.header:sub(HEADER_SIZE+10)
@@ -492,14 +515,14 @@ end
 -- @return rows table containing row tables as decoded by <code>decodeDataPackets</code>
 function sqlQuery( socket, query )
 
-  local catch = function() socket:close() stdnse.print_debug("sqlQuery(): failed") end
+  local catch = function() socket:close() stdnse.debug1("sqlQuery(): failed") end
   local try = nmap.new_try(catch)
   local packetno = 0
   local querylen = query:len() + 1
   local packet, packet_len, pos, header
   local status, fields, field_count, rows, rs
 
-  packet = bin.pack("ICA", querylen, Command.Query, query )
+  packet = bin.pack("<ICA", querylen, Command.Query, query )
 
   --
   -- http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Result_Set_Header_Packet
