@@ -3,7 +3,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2016 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2017 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -52,7 +52,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: engine_select.c 36171 2016-08-22 19:15:13Z tudor $ */
+/* $Id: engine_select.c 36816 2017-06-19 22:58:23Z dmiller $ */
 
 #ifndef WIN32
 #include <sys/select.h>
@@ -208,6 +208,9 @@ int select_iod_modify(struct npool *nsp, struct niod *iod, struct nevent *nse, i
   iod->watched_events |= ev_set;
   iod->watched_events &= ~ev_clr;
 
+  ev_set |= EV_EXCEPT;
+  ev_clr &= ~EV_EXCEPT;
+
   sd = nsock_iod_get_sd(iod);
 
   /* -- set events -- */
@@ -233,7 +236,7 @@ int select_iod_modify(struct npool *nsp, struct niod *iod, struct nevent *nse, i
 
   /* -- update max_sd -- */
   if (ev_set != EV_NONE)
-    sinfo->max_sd = MAX(sinfo->max_sd,sd);
+    sinfo->max_sd = MAX(sinfo->max_sd, sd);
   else if (ev_clr != EV_NONE && iod->events_pending == 1 && (sinfo->max_sd == sd))
     sinfo->max_sd--;
 
